"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var util_1 = require("./util");
/**
 * 判断是否同源请求
 * @param requestURL 请求的URL
 */
function isURLSameOrigin(requestURL) {
    var parsedOrigin = resolveURL(requestURL);
    return (parsedOrigin.protocol === currentOrigin.protocol && parsedOrigin.host === currentOrigin.host);
}
exports.isURLSameOrigin = isURLSameOrigin;
// 通过创建一个a标签，设置a标签的href属性为某个url，直接从a标签的protocol和host属性中读书协议、域名端口
// protocol表示协议如，'http:'
// host表示域名端口如，'127.0.0.1:8080'
var urlParsingNode = document.createElement('a');
var currentOrigin = resolveURL(window.location.href);
function resolveURL(url) {
    urlParsingNode.setAttribute('href', url);
    var protocol = urlParsingNode.protocol, host = urlParsingNode.host;
    return {
        protocol: protocol,
        host: host
    };
}
function encode(val) {
    // 将URL内容进行编码，但是将以下符号转换回来不编码
    // 带字母的部分正则加i标记不区分大小写
    return encodeURIComponent(val)
        .replace(/%40/g, '@')
        .replace(/%3A/ig, ':')
        .replace(/%24/g, '$')
        .replace(/%2C/ig, ',')
        .replace(/%20/g, '+')
        .replace(/%5B/ig, '[')
        .replace(/%5D/ig, ']');
}
/**
 * 构建get请求的url函数
 * 构建的规则：
 * 1、如果参数属性，对应的值是数组，如params:{foo: ['bar', 'baz']}，那么在url中的形式为：xxxx?foo[]=bar&foo[]=baz
 * 2、如果参数属性，对应的值是对象，则为stringify后的字符串，并且经过编码
 * 3、url中带有哈希值，即'#'符号，忽略url中'#'符号后面的内容
 * 4、url中若已有参数，则params中的参数跟在后面，否则url末尾加上'?'，并且跟上参数
 * 5、特殊的符号如'@'、':'、'$'、'['、']'、','转码回来，不显示编码后的代码，空格符用'+'号替代
 * 6、如果参数的属性，对应的值是日期对象，则值用toISOString()转换成字符串
 * 7、如果参数的属性，是null或者undefined，则url中不带上这个参数
 * @param url axios请求的url地址
 * @param params 请求的参数
 */
function buildURL(url, params, paramsSerializer) {
    if (!params) {
        return url;
    }
    var serializedParams;
    if (paramsSerializer) {
        serializedParams = paramsSerializer(params);
    }
    else if (util_1.isURLSearchParams(params)) {
        serializedParams = params.toString();
    }
    else {
        var parts_1 = [];
        Object.keys(params).forEach(function (key) {
            var val = params[key];
            // 如果属性的值是null，或者undefined，url中不会带上这个参数，直接跳过
            if (val === null || typeof val === 'undefined') {
                return;
            }
            var values = [];
            // 如果属性的值是一个数组，那么url中key的后面要跟上'[]'
            if (Array.isArray(val)) {
                values = val;
                key += '[]';
            }
            else {
                // 如果属性值不是数组，那么将它变成数组，方便处理
                values = [val];
            }
            values.forEach(function (val) {
                if (util_1.isDate(val)) {
                    val = val.toISOString();
                }
                else if (util_1.isPlainObject(val)) {
                    val = JSON.stringify(val);
                }
                parts_1.push(encode(key) + "=" + encode(val));
            });
        });
        serializedParams = parts_1.join('&');
    }
    // 如果上面的处理结果是空数组，那么join后是空字符串，不处理
    // 同时如果url里有哈希值，即myUrl#hash形式，哈希后面的是页面内锚点或跳转，不需要在请求url中跟上
    if (serializedParams) {
        var markIndex = url.indexOf('#');
        if (markIndex !== -1) {
            url = url.slice(0, markIndex);
        }
        // 如果这个url已经带有参数，那么它本身带有'?'，后面加上'&'并拼接
        // 否则，先加上'?'，并拼接
        url += (url.indexOf('?') === -1 ? '?' : '&') + serializedParams;
    }
    return url;
}
exports.buildURL = buildURL;
/**
 * 判断一个url是否是绝对地址
 * @param url URL地址
 */
function isAbsoluteURL(url) {
    // 正则含义：
    // 绝对地址的url，以类似'http:'或'https:'开头，后面跟两个斜线'//'；也可以直接两个斜线开头
    // 正则匹配字母开头，后面可以是0个或多个字母、数字、加号'+'、减号'-'、点号'.'，再跟冒号':'。这部分可能有也可能没有，所以跟问号表示0或1个
    // 斜线需要转义
    // 需要忽略大小写，正则表达式跟i标识符
    return /^([a-z][a-z\d\+\-\.]*:)?\/\//i.test(url);
}
exports.isAbsoluteURL = isAbsoluteURL;
/**
 * 将基础URL和相对路径URL拼成完整的URL
 * @param baseURL 基础URL
 * @param relativeURL 相对路径URL
 */
function combineURL(baseURL, relativeURL) {
    // 拼接时，删除baseURL末尾的斜线'/'，以及relativeURL开头的斜线'/'，两个再以斜线'/'拼接
    return relativeURL ? baseURL.replace(/\/+$/, '') + '/' + relativeURL.replace(/^\/+/, '') : baseURL;
}
exports.combineURL = combineURL;
//# sourceMappingURL=url.js.map